@@ -9,30 +9,34 @@ void writeBUSHeader(std::ofstream &out, int bclen, int umilen) {
   out.write((char*)(&umilen), sizeof(umilen));
   std::string header_text = "BUS file produced by kallisto";
   uint32_t len = header_text.size();
-  out.write((char*)(&len),sizeof(len));  
+  out.write((char*)(&len),sizeof(len));
   out.write(header_text.c_str(), len);
 }
 
-void writeBUSData(std::ofstream &out, const std::vector<BUSData> &bv) {
+size_t writeBUSData(std::ofstream &out, const std::vector<BUSData> &bv, MinCollector* tc) {
+  size_t total = 0;
   for (const auto &b : bv) {
     if (b.ec != -1) { // maybe keep non-mapping reads ?!?
       out.write((char*)(&b), sizeof(b));
+      ++total;
+      if (tc != nullptr) ++((*tc).counts[b.ec]);
     }
   }
+  return total;
 }
 
 void writeBUSMatrix(const std::string &filename,
-                    std::vector<std::vector<std::pair<int,int>>> &data, int cols) {
+                    std::vector<std::vector<std::pair<int,int> > > &data, int cols, std::vector<int> mapping) {
   std::ofstream of;
   of.open(filename.c_str(), std::ios::out | std::ios::binary);
   writeBUSHeader(of, BUSFORMAT_FAKE_BARCODE_LEN, 1);
-  if (!data.empty()) { // reduntant  
+  if (!data.empty()) { // reduntant
     for (size_t j = 0; j < data.size(); j++) {
       const auto &v = data[j];
       for (size_t i = 0; i < v.size(); i++) {
         if (v[i].second != 0 && v[i].first != -1) {
           BUSData b;
-          b.barcode = j;
+          b.barcode = mapping[j];
           b.flags = 0;
           b.UMI = -1;
           b.ec = v[i].first;
@@ -43,4 +47,4 @@ void writeBUSMatrix(const std::string &filename,
     }
   }
   of.close();
-}
\ No newline at end of file
+}
