@@ -3,13 +3,14 @@
 
 #include <fstream>
 #include <vector>
+#include "MinCollector.h"
 #include "BUSData.h"
 
 const uint32_t BUSFORMAT_FAKE_BARCODE_LEN = 16;
 
-void writeBUSData(std::ofstream &out, const std::vector<BUSData> &bv);
+size_t writeBUSData(std::ofstream &out, const std::vector<BUSData> &bv, MinCollector* tc = nullptr);
 void writeBUSHeader(std::ofstream &out, int bclen, int umilen);
 void writeBUSMatrix(const std::string &filename,
-                    std::vector<std::vector<std::pair<int,int>>> &data, int cols);
+                    std::vector<std::vector<std::pair<int,int>>> &data, int cols, std::vector<int> mapping);
 
-#endif // KALLISTO_BUSTOOLS_H
\ No newline at end of file
+#endif // KALLISTO_BUSTOOLS_H
